<?php
/*
 *  Class: GalmetaPostResource
 *  This represents a single instance of a GalmetaPost Resource. I has all the methods and properties
 *  necessary to display and operate on the resource.
 * 
 *  License:
 *  Galmeta Post is a Open Source Content Management System
 *  Copyright (C) 2007  Galmeta LLC
 *  
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version 2
 *  of the License, or (at your option) any later version.
 *  
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.    
 */
class GalmetaPostResource extends Hookable {

    /*
     * Property: galmetapost
     * Reference to the galmetapost request object created for this request session.
     * 
     * See Also:
     *  <GalmetaPostRequest>
     */
    var $galmetapost = null;

    /*
     * Property: uri
     * String, the raw unaltered request for this resource
     */
    var $uri = null;

    /*
     * Property: title
     * String, the pretty version of <name>, capitalized and spaced out.
     * 
     * SeeAlso:
     *  <makeTitle()>
     */
    var $title = '';

    /*
     * Property: context
     * String, the context of this resource in relation to another possible resource
     * which has created this one via a plugin. For instance, if a page named
     * "dogs" uses <GalmetaPostPlugin_include> or {include:} to include another
     * page named "boxer", then the boxer resource will have a context of "dogs".
     */
    var $context = null;
    
    /*
     * Property: resource
     * String, the full path resource which is used to match against the name field
     * in the database. It is parsed by <parse_uri()>.
     * 
     * See Also:
     *  <parse_uri()>
     */
    var $resource = null;

    /*
     * Property: name
     * String, the name portion of the <resource>, with the path removed. 
     * 
     * Example:
     *  A resource of my/pet/dog will have a name of dog.
     * 
     * See Also:
     *  <parse_uri()>
     */
    var $name = null;

    /*
     * Property: path
     * String, path portion of the <resource>.
     * 
     * Example:
     *  A resource of my/pet/dog will have a path of my/pet
     */
    var $path = null;

    /*
     * Property: list
     * Boolean, true if this resource was requested as a list, essentially
     * checks for an ending slash "/".
     * 
     * Example:
     *  A request of my/pet/dog/ will set this to true, however a request of
     *  my/pet/dog will set this to false.
     * 
     * See Also:
     *  <parse_uri()>
     */
    var $list = null;

    /*
     * Property: view
     * String, if this resource is being requested using a different view,
     * the value of that view would be found here. Valid views can be found
     * in the global define <VALID_GALMETAPOST_VIEWS>.
     *  
     * Example:
     *  A request of my/pet/dog!edit will have a view of "edit". A request of
     *  my/pet/dog!v will have a view of "v".
     * 
     * See Also:
     *  <parse_uri()>
     */
    var $view = null;

    /*
     * Property: parameters
     * String, additional text found in the request after a !view but before
     * an & amprsign, the query string portion.
     * 
     * Example:
     *  A request of my/pet/dog!v23&offset=10 will set this to "23". A request
     *  of blog!rss2 will set this to "2".
     * 
     * See Also:
     *  <parse_uri()>
     */
    var $parameters = null;

    /*
     * Property: content
     * String, when this resource is pulled from the database, the contents of
     * of the content field will be placed here.
     * 
     * See Also:
     *  <get()>
     */
    var $content = null;

    /*
     * Property: edited_on
     * String, this will have the last time this resource has been edited. Or
     * the contents of the "edited_on" database field.
     * 
     * See Also:
     *  <get()>
     */
    var $edited_on = null;

    /*
     * Property: edited_by
     * String, this will have the last person who edited this resource. Or
     * the contents of the "edited_by" database field. Will be in the form
     * "username@ipaddress" such as "guest@127.0.0.1".
     * 
     * See Also:
     *  <get()>
     */
    var $edited_by = null;

    /*
     * Property: locked
     * String, this will have the timestamp of locked database field. It
     * can be used to check if an when the resource was last locked.
     * 
     * See Also:
     *  <get()>
     */
    var $locked = null;

    /*
     * Property: version
     * Integer, this will have the current version of the resource. Everytime an
     * update is made to the resource this will be incremented.
     * 
     * See Also:
     *  <get()>
     *  <GalmetaPostRequest::updateResource()>
     */
    var $version = null;

    /*
     * Property: can_read
     * Boolean, true if the requesting user has read rights to this resource
     * 
     * See Also:
     *  <GalmetaPostRequest::getMethodAccess()>
     */
    var $can_read = false;
    
    /*
     * Property: can_create
     * Boolean, true if the requesting user has create (insert) rights to this resource
     * 
     * See Also:
     *  <GalmetaPostRequest::getMethodAccess()>
     */
    var $can_create = false;

    /*
     * Property: can_update
     * Boolean, true if the requesting user has update rights to this resource
     * 
     * See Also:
     *  <GalmetaPostRequest::getMethodAccess()>
     */
    var $can_update = false;

    /*
     * Property: can_delete
     * Boolean, true if the requesting user has delete rights to this resource
     * 
     * See Also:
     *  <GalmetaPostRequest::getMethodAccess()>
     */
    var $can_delete = false;

    /*
     * Property: can_execute
     * Boolean, true if the requesting user has the right to run plugins found in
     * this resource.
     * 
     * See Also:
     *  <GalmetaPostRequest::getMethodAccess()>
     */
    var $can_execute = false;

    /*
     * Property: rewrote
     * String, non-blank if the resource matched a rewrite pattern in the registry
     * and was rewritten as a result. The pattern is stored in this var. 
     */
    var $rewrote = false;

    /*
     * Property: rewrite_vars
     * Array, If a rewrite in the registry rule matches this resource's uri, any
     * extracted are placed into this array. 
     * 
     * Example:
     * > uri: /forum/43/23
     * > pattern: /forum\/([^\/]+)\/([^\/]+)/
     * > replace: forum?id=$1&parent=$2
     * > rewrite_vars: array( 'id'=>'43','parent'=>23)
     */
    var $rewrite_vars = array();

    /*
     * Property: exists
     * Boolean, true if this resource exists in the database. Can only be checked
     * after using <get()>.
     * 
     * See Also:
     *  <get()>
     */
    var $exists = null;

    /**
    * Method: GalmetaPostResource
    * Constructor for resource.
    *
    * Parameters:
    *   gp - <GalmetaPostRequest> object
    *   uri - String, resource uri to load retrieve, most commonly passed from browser url.
    *   context - Optional Object, <GalmetaPostResource> object that represents the context of this resource. This is used
    *   when there's sub-requests using the {include:} plugin to include other resources such as menus, lists, headers, etc.
    *   apply_rewrite - Optional boolean, Control if uri rewrite rules are applied if found in registry. Defaults to true.
    */
    function GalmetaPostResource( &$gp, $uri, $context = null, $apply_rewrite = true )
    {
        $this->galmetapost =& $gp; 
        $this->uri = $uri;        
        $this->context = $context;

        $this->initializeHookPoints();
    
        $request =& $this->galmetapost->getRequest();
    
        $info = parse_uri( $uri, $context, (($request)?$request->getVar('resource'):null));

        foreach ( $info as $name => $value )
        {
            $this->{$name} = $value;
        }

        //if base request, use galmetapost->name as resource
        if ( $this->resource == '' )
        {
            $this->resource = $this->galmetapost->getVar('name');
            $this->name = $this->galmetapost->getVar('name');
        }

        //make a pretty title//
        $this->title = prettytext( $this->name );

        //get the user's access rights of this resource//
        $access = $this->galmetapost->call('getMethodAccess', $this->resource );

        $this->can_create = $access['c'];
        $this->can_read = $access['r'];
        $this->can_update = $access['u'];
        $this->can_delete = $access['d'];
        $this->can_execute = $access['x'];

        //apply rewrites to $this->resource//        
        if ( $apply_rewrite && $rewrites = $this->galmetapost->call('registry','rewrites//') )
        {
            foreach ( $rewrites as $rewrite )
            {
                $pattern = str_replace( 'rewrites/', '', $rewrite['name'] );
                $test = preg_replace( '/'.$pattern.'/' ,$rewrite['content'], $this->resource );

                if ( $test != $this->resource )
                {
                    $this->rewrote = str_replace('/','\\/',$rewrite['name']);
                    $info = parse_url( $test );
                    $this->resource = $info['path'];
                    $query = array();
                    parse_str( $info['query'], $query );
                    foreach ( $query as $name => $value )
                    {
                        $this->rewrite_vars[$name] = $value;
                    }
                }
            }
        }   
    }

    /*
        Method: initializeHookPoints
        This function creates a series of hookpoints in the GalmetaPostResource object using the hooks pattern so
        plugins can insert their own processing at various points. Plugins can insert their own functions via
        the <AbstractGalmetaPostPlugin::registerPluginHook()> function, or using <GalmetaPostRequest::registry()> function.
        This queries the registry database for all names matching 'hooks/%' and hooks based on record content.        
    */    
    function initializeHookPoints(){

        static $hooks = null;
        
        if ( $hooks )
        {
            $this->_hooks = $hooks;
            return;
        }

        $this->hook('exists');
        $this->hook('lock');
        $this->hook('unlock');
        $this->hook('isLocked');
        $this->hook('get');
        $this->hook('getList');
        $this->hook('render');
        $this->hook('getRender');
        $this->hook('plugin');
        $this->hook('checkAccess');
        $this->hook('getCount');
        
        //load registered hooks//
        $hooks = $this->galmetapost->call('registry','hooks/GalmetaPostResource//', null);

        if ( $hooks )
        {
            foreach ( $hooks as $hook )
            {
                $specs = explode( '/', $hook['name'] );
                $this->hook ( $hook['content'] , $specs[3], $specs[2] );
            }       
        }
        
        $hooks = $this->_hooks;
    }


    /*
        Method: getVar
        General var getter for GalmetaPostResource

        Parameters: 
            var - name of the property to get
            def - default value to return if none-existant

        Return
            mixed - value of $this->var
    */
    function getVar( $var, $def = null )
    {
        if ( isset($this->{$var}) && $this->{$var} !== null )
        {
            return $this->{$var};
        }
        else
        {
            return $def;
        }
    }

    /*
        Method: setVar
        General var setter for GalmetaPostResource

        Parameters: 
            var - name of the property to set
            val - value to set

        Return
            mixed - value of $this->var
    */
    function setVar( $var, $val )
    {
        return $this->{$var} = $val;
    }

   /*
    * Methods: exists
    * Checks to see if the galmetapost resource exists. This function does not
    * check access. It assumes it's been done already.
    *
    * Return:
    *   mixed - False when doesn't exist, select results (hash) if it does exist.
    */
   function exists()
   {
      $resource = $this->galmetapost->call('findByName', $this->resource );

      if ( $resource )
      {
         return $resource;
      } 
      elseif ( $this->list ) 
      {
         return array('name'=>$this->resource,'content'=>'{ls:}');
      } 
      else 
      {
         return false;
      }
   }


   /*
    * Method: lock
    * Locks a resource by timestamping the locked column. This function
    * is secure and the request is sanitized.
    * This function is secure.
    * 
    * Return:
    *   boolean - true if successful, false otherwise
    */
    function lock()
    {
        $this->get();
        if ($this->checkAccess('u') && $this->exists )
        {
            $this->galmetapost->call('lock', $this->resource );
            return true;
        }
        
        return false;
    }


   /*
    * Method: unlock
    * Unlocks a resource by nullifying the column. This function
    * is secure and the request is sanitized.
    * This function is secure.
    * 
    * Return:
    *   boolean - true if successful, false otherwise
    */
    function unlock()
    {
        $this->get();
        if ($this->checkAccess('u') && $this->exists )
        {
            $this->galmetapost->call('unlock', $this->resource );
            return true;         
        }
        
        return false;
    }

   /*
    * Method: isLocked
    * Checks to see if this resource is considered locked by looking at the
    * system's timeout policy. <get()> must be called prior to calling this.
    * 
    * Return:
    *   boolean - true if locked, false if not. 
    */
    function isLocked()
    {
        return ( $this->locked != 0 && ((time() - strtotime($this->locked))/60) < $this->galmetapost->getVar('lock_timeout'));
    }

   /*
    * Method: render
    * Apply parsing rules and execute plugins to a string, usually $this->content. This
    * method has a static var $INF used to protect against infinite loops.
    *
    * Parameters:
    *   str - Optional, string of data to parse and render, defaults to $this->content.
    * 
    * Return:
    *   string - Rendered string
    */
    function render( $str = null )
    {
        //infinite loop protection//
        static $INF = array();
        
        $str = ($str ? $str : $this->content );
        
        if ( in_array( $this->resource , $INF ) )
        {
            return '<span class="error">Too much recursion</span>';
        }
        
        $pattern = array();
        $replace = array();
        
        array_push( $INF, $this->resource );
        
        //get content syntax rules//
        list($p,$r) = $this->galmetapost->call('rules');
        
        $pattern = array_merge($pattern,$p);
        $replace = array_merge($replace,$r);
        
        //execute plugins//
        $pattern[] = '/\{([^\}\s]+)\:(\s([^\}]+))?\}/e';         
        $replace[] = ( $this->can_execute ? '$this->plugin(\'$1\',\'$3\',$this);' : '$this->galmetapost->getAccessDenied( \'$1\' )' );
      
        $str = preg_replace($pattern, $replace, $str);
           
        
        array_pop( $INF );
        
        return $str;
    }

    /*
     * Method: checkAccess 
     * Check the access right (crudx) of the current user.
     *
     * Parameters:
     *  method - What they plan to do to the resource [ w - write[cud] r - read x - execute c - create u - update d - delete ]
     *  username - Optional, Username of one who's to check access against. Defaults to $_SESSION['username']
     * 
     * Return:
     *  boolean - True if access is granted, false if otherwise.
     */   
    function checkAccess( $method, $username = null )
    {
        return $this->galmetapost->checkAccess( $this->resource, $method, $username );
    }

   /*
    * Method: getRender
    * Get the resource from the database and parses content for plugins,rules.
    *
    * @param string $part Part of the resource to return, defaults to content.
    * @return string, part of the resource requested. Blank
    */
   function getRender( $part = 'content' )
   {
       $this->call('get', $part );

       if ( $part == 'content' )
       {        
           return $this->call('render');
       } 
       else 
       {
           return $this->{$part};
       }
   }


   /*
    * Method: get
    * Loads the resource from the database. Loads this resource object with database 
    * fields and other flagging specs.
    * 
    * Parameters:
    *   part - Part of the resource to return, defaults to content. Values include:
    *       [content|id|edited_on|edited_by|locked|version]
    * 
    * Return:
    *   string - part of the resource requested. Blank if none.
    */
   function get( $part = 'content' )
   {
        //check access, return immediatly if denied//
        if ( !$this->can_read && $part == 'content' )
        {
            $this->content = $this->galmetapost->call('getAccessDenied', $this->resource );
            return $this->content;
        }

        $rtn = '';

        //if exists, get it//
        if ( $result = $this->exists() )
        {
            $this->exists = true;

            foreach ( array( 'id', 'content', 'edited_on', 'edited_by', 'locked', 'version' ) as $field )
            {
                $this->{$field} = $result[ $field ];
            }

            $rtn = $result[ $part ];
        }
        else //doesn't exist
        {
            $this->exists = false;

            //doesn't exist, check for page_not_found resource//
            if ( $this->resource != 'page_not_found' && $part == 'content' 
                     && ($pnf = $this->galmetapost->locateCore('page_not_found')) 
                     && $this->view == null )                 
            {
                $nr = new GalmetaPostResource( $this->galmetapost, '/' . $pnf );
                $this->content = $nr->get('content');
                $rtn = $this->content;
            }
            
            //check for rogue rewrite to remove//
            if ( $this->rewrote )
            {
                $this->galmetapost->registry( $this->rewrote, '');
            }
        }
        
        return $rtn;
   }

    /*
     * Method: getList
     * Gets a list of sub-resources or versions located beneath this resource. If this
     * isn't set to a <list> resource, or a version <view>, it will just do a general
     * list of any resources named similar, ie /dir/myfile will look for anything with that
     * name in it. 
     *
     * Parameters:
     *  sort - Optional String, Field to sort on. Defaults to "edited_on".
     *  order - Optional String, Order to sort by. Defaults to "DESC".
     *  offset - Optional int, Record offset of the list to start at. Defaults to 0.
     *  limit - Optional int, Record limit of the list. Defaults to 30.
     *  checkaccess - Optional boolean, Remove records not accessible by user. Defaults to true
     *  core_resources - Include system files in query. Defaults to false
     * 
     * Return:
     *  Array - 2 Dimensional Associative arrays of the results of the query. (name, content, edited, edited_ip)
     */
    function getList( $sort = null, $order = null, $offset = null, $limit = null, $checkaccess = true, $core_resources = false )
    {
        return $this->galmetapost->call('findAllByName', $this->resource . ($this->list?'/':''), $sort, $order, ($this->view == 'v'), $offset, $limit, $checkaccess, $core_resources );
    }
    
    /**
     * Query the database for a count of child resources
     *
     * @param string $path The name of the galmetapost content to query for with trailing slash.
     * @param boolean $checkaccess Use access control to get count.
     * @return int count of matches
     */
    function getCount( $checkaccess = true )       
    {
        return $this->galmetapost->call('getCount', $this->resource . ($this->list?'/':''), ($this->view == 'v'), $checkaccess );
    }
    
    /**
    * Executes a galmetapost plugin, loading one if nessesary, checking for execution rights
    *
    * @param string $name Name of the plugin to call.
    * @param array $string Optional comma delimited parameter list.
    * @return string Results of plugin call.
    */
   function plugin( $name, $param='', $context = null )
   {    
        if ( $this->galmetapost->checkAccess( "{{$name}:}", 'x') )
        {      
            if ( $p =& $this->galmetapost->call ('getPlugin', $name) ) 
            {
                if ( $this->view == 'admin' && method_exists( $p, 'admin' ) )
                {
                    if ( $this->galmetapost->checkAccess( "{{$name}:}", 'c') )
                    {
                        return ( $p->call('admin', $param, $context ));
                    }
                    else
                    {
                        return ("<span class=\"error\">Access denied to administrate <strong>{$name}</strong> plugin!</span>");
                    }
                }
                else
                {                         
                    return ( $p->call( 'run', $param, $context ) );
                }
            } 
            else 
            {
                return ("<span class='error'>{$name} plugin not installed!</span>");
            }
        } 
        else 
        {
            return ("<span class=\"error\">Access denied to <strong>{$name}</strong> plugin!</span>");
        }
   }
   
    /*
     *  Method: getPlugin
     *  Retrieves a plugin from the cached internal array or the plugins folder, method
     *  override in Hookable
     * 
     *  Parameters:
     *      plugin - String, name of the plugin to get.
     * 
     *  Return:
     *      string - Reference to the loaded plugin or false if not found.
     */
    function &getPlugin( $plugin )
    {
        return $this->galmetapost->getPlugin( $plugin );
    }       

    /*
     * Method: uninstallPlugin
     * Override method for Hookable class to perform any potential cleanup
     * if a registered hook results in a call to a non-existant object, in this
     * case it means the plugin has been uninstalled.
     * 
     * Parameters:
     *  name - String, name of the plugin to uninstall.
     */
     function uninstallPlugin( $name )
     {
        $this->galmetapost->uninstallPlugin( $name );                                    
     }
   
}

?>