<?php
/*
 * Class: HookCondition
 * The hook condition class is an abstract class used to define special case return
 * values in conjunction with the <Hookable::call> method. Nothing too special about it
 * but it allows for a custom namespace for these return conditions.
 */
class HookCondition
{
    /*
     * Property: break
     * Boolean, if this return condition requires a break in the call chain.
     */
    var $break = false;
    
    /*
     * Property: params
     * Array, usually a holder for a parameter override case.
     */
    var $params = array();

    /*
     * Property: params
     * mixed, usually a holder for a return override case.
     */
    var $rtn = null;   
        
    /*
     * Method: isBreak
     * Accessor function for <HookCondition::break>.
     * 
     * Return:
     *  Boolean - if this condition requires a break in execution.
     */        
    function isBreak()
    {
        return $this->break;
    }
    
}

/*
 * Class: HookBreakCondition
 * A special hook return condition requiring the executing call to return immediatly
 * with the value passed to the HookBreakCondition constructor.
 */
class HookBreakCondition extends HookCondition
{
    function HookBreakCondition( $return )
    {
        $this->rtn = $return;
        $this->break = true;
    }   
    
    function handle( &$params, &$return )
    {
        $return = $this->rtn;
    }    
}

/*
 * Class: HookPassCondition
 * A special hook return condition changing the value of previously passed parameters 
 * in the executing call chain. The parameters to override with are passed to the 
 * constructor. 
 */
class HookPassCondition extends HookCondition
{
    function HookPassCondition()
    {
        $this->params = func_get_args();
    }
    
    function handle( &$params, &$return )
    {
        $params = $this->params;
    }
}


?>