<?
/**
 * Class: GalmetaPostPlugin_blog
 * The blog plugin creates a spot where a blog or podcast can be started, displayed and managed.
 * The blog plugin can also handle blog comments. The default page the comments are
 * saved under is /comments. If you want to allow guests to comment on posts, add
 * read, create and execute privileges to guests on "comments/*".
 * 
 * Plugin Parameters:
 *  path - path to use as the blog's home
 *  comments - boolean, if comments should be allowed on the blog posts.
 *  as - string, how to display the blog posts. [full|link|summary].
 *  pagination - boolean, if pagination should be used. 
 *  limit - How many posts to display on a page.
 *  offset - How many posts to skip in the listing 
 *  sort - What database field to sort on
 *  order - what order to use [ASC|DESC]
 *  post_link - boolean, 
 * 
 * CSS Layout:
 * <.blog>
 *  content
 *  <.date>date</.date>
 *  <.blog-comments>
 *      <#blog-comments-N .blog-comments-N>
 *          <.blog-comment>
 *              <.blog-comment-body>content</.blog-comment-body>
 *              <.blog-comment-date>date</.blog-comment-date>
 *          </.blog-comment>
 *      </#blog-comments-N .blog-comments-N>
 *  </.blog-comments>
 * </.blog>
 * 
 */
class GalmetaPostPlugin_blog extends AbstractGalmetaPostPlugin {

    function GalmetaPostPlugin_blog( &$G ){
        $this->name = 'Blog';
        $this->version = '1.0';
        $this->description = 'Simple Blogging/Podcasting plugin.';
        $this->dependencies = array(
            'famfamfam Icons' => array('path'=>'_lib/famfamfam'),
            '1pixelout Audio-Player' => array('path'=>'_lib/audio-player')            
            );
        $this->author = 'Bryan English <bryan@galmeta.com>';
        $this->link = 'http://galmeta.com/post/';
        $this->copyright = 'Copyright &copy; 2007 Galmeta LLC';
        $this->usage = array(
            'path' => array(
                'default' => null,
                'description' => 'Absolute url path to save and aggregate the blog data. Defaults to current request. '),
            'limit' => array(
                'default' => 10,
                'description' => 'Number of latest postings to pull. Defaults to 10.'),
            'offset' => array(
                'default' => 0,
                'description' => 'Number of latest postings to pull. Defaults to 10.'),
            'as' => array(
                'default' => 'full',
                'description' => 'Type of display (full, link, summary, calendar). Defaults to full.'),
            'post_link' => array(
                'default' => true,
                'description' => 'Toggles the "Add Post" link to the top of the entries for users with write permissions.'),
            'pagination' => array(
                'default' => true,
                'description' => 'Toggles the pagination navigation links.'),
            'comments' => array(
                'default' => false,
                'description' => 'If to allow comments. The comments folder should have create access rights.'),
            'sort' => array(
                'default' => 'edited_on',
                'description' => 'Sort field to sort the blog items on.'),
            'order' => array(
                'default' => 'DESC',
                'description' => 'Ordering to sort with')
        );
        
        parent::AbstractGalmetaPostPlugin( $G );
    }
  
    function run( $params, &$resource ){
   
   		//get set with all paramters//
        $params = $this->getParameters( $params );

        $params['as'] = preg_replace('/[^a-z0-9]/i','',$params['as']);

		//look for pagination query overrides//
        $params['sort'] =  epick( slash($_GET['sort']), $params['sort'] );
        $params['order'] = epick( slash($_GET['order']), $params['order'] );
        $params['offset'] = intval( epick( slash($_GET['offset']), $params['offset']));      

/* do i need this?
        if ( !$resource->getVar('resource') ) {
            $resource->resource = $this->galmetapost->request;
        }
*/
        $blog = new GalmetaPostResource( $this->galmetapost, $params['path'], $resource->getVar('resource') );
        $blog->get();

        $newpost = date('Y/m/d/His');

        if ( $blog->getVar('can_create') && $params['post_link'] )
        {
            if ( is_string( $params['post_link'] ) )
            {
                $title = $params['post_link'];
            } 
            else 
            {
                $title = 'Add '. ($params['as'] == 'comment'?'Comment':'Post');
            }

            $this->set('post_link', '<a href="'. $this->galmetapost->getVar('url_prefix') . $blog->getVar('resource') . '/' . $newpost .'!edit" class="bloglink">'. $title .'</a>');
        }

		$path = $blog->getVar('resource') . '%/____/__/__/______';
        $blogs = $this->galmetapost->call( 'findAllByName', $path, $params['sort'], $params['order'], $params['offset'], $params['limit'] );   
        $total = $this->galmetapost->call( 'getCount', $path );

        if ( !file_exists($this->galmetapost->getVar('path') . '_ext/plugins/blog/'. $params['as'] .'.thtml') ) 
        {
            return "<span class='error'>Blog type doesn't exist.</span>";
        }

        if ( $params['comments'] && $params['as'] == 'full')
        {
            $this->galmetapost->call( 'HTMLInclude', '_lib/mootools.js');
            $this->galmetapost->call( 'HTMLInclude', '_ext/plugins/blog/blog.js');
        }

        if ( $blogs )
        {
            $max = count($blogs);
            for ( $cnt = 0; $cnt < $max; $cnt++ )
            {
                $matches = array();

                //analyze the galmetapost resource name//
                if (!preg_match ( '/'. str_replace('/','\\/',$blog->getVar('resource')) .'.*?(\d\d\d\d)\/(\d\d)\/(\d\d)\/(\d\d)(\d\d)(\d\d)/', $blogs[$cnt]['name'], $matches ))
                {
                    unset( $blogs[$cnt] );
                    continue;
                }

				//make a time from page name//
                $time = "{$matches[1]}-{$matches[2]}-{$matches[3]} {$matches[4]}:{$matches[5]}:{$matches[6]}";
                $time = date( $this->galmetapost->getVar('date_format'), strtotime($time) );

				$blogs[$cnt]['title'] = h123456( $blogs[$cnt]['content'], $time );

                $blogs[$cnt]['date'] = $time;
                $blogs[$cnt]['link'] = $this->galmetapost->getVar('url_prefix') . $blogs[$cnt]['name'];

                if ( $params['comments'] )
                {
                    $blogs[$cnt]['comment_link'] = 'comments/' . $blogs[$cnt]['name'] . '/~!edit';
                    $comments = new GalmetaPostResource( $this->galmetapost, '/comments/' . $blogs[$cnt]['name']);
                    $blogs[$cnt]['comment_count'] = $comments->getCount();
                }
            }
        } 
        else 
        {
            $blogs = array();
        }

        $this->set( 'blogs', $blogs );
        $this->set( 'total', $total );
        $this->set( 'canCreate', $blog->getVar('can_create') );
        $this->set( 'canUpdate', $blog->getVar('can_update') );
        $this->set( 'comments', $params['comments'] );
        $this->set( 'prefix', $this->galmetapost->getVar('url_prefix') );
        $this->set( 'url', $resource->getVar('resource') );
        $this->set( 'params', $params );
        
        //get content syntax rules//
        list($pattern,$replace) = $this->galmetapost->call('rules');
        $content = preg_replace($pattern, $replace, $this->renderTemplate ( $params['as'] ));
                        
        return $content;

   }

} 
?>