<?

/**
 * Class: GalmetaPlugin_rss
 * RSS import plugin. Fetches a external rss feed and displays it. This plugin requires
 * the Magpieress php library. It should be located in "_lib/magpierss". Additionally,
 * a cache directory should be made to save rss feeds. Create a directory "_cache/magpierss".
 *
 * Plugin Parameters:
 *  url - string, url to fetch or multiple urls seperated by spaces.
 *  limit - number of items to display.
 *  title - optional, string of feeds title.
 *  style - optional, template to use to display the rss items defaults to base. [basic|full]
 */

class GalmetaPostPlugin_rss extends AbstractGalmetaPostPlugin {

    function GalmetaPostPlugin_rss( &$G ){
		$this->name = 'RSS';
		$this->version = '1.0';
		$this->description = 'Imports and displays a RSS feed. Requires the MagPieRSS library';
		$this->author = 'Bryan English <bryan@galmeta.com>';
        $this->dependencies = array(
            'MagPieRSS' => array('path'=>'_lib/magpierss')
            );
		$this->link = 'http://galmeta.com/post/';
		$this->copyright = 'Copyright &copy; 2007 Galmeta LLC';
        $this->usage = array(
            'url' => array(
                'default' => null,
                'description' => 'URL to the rss feed(s) to grab, separated by spaces. Required.'),
            'title' => array(
                'default' => null,
                'description' => 'Title of the rss feed. Defaults to first rss feeds channel name.'),
            'limit' => array(
                'default' => 10,
                'description' => 'Number of topics to grab and display. Defaults to 10.'),
            'style' => array(
                'default' => 'basic',
                'description' => 'What template to use to display the feed.')
        );
        
        parent::AbstractGalmetaPostPlugin( $G );
    }

	function run( $params )
    {	        
        $params = $this->getParameters( $params );

        if ( empty($params['url']) )
        {
            return '';
        }

        $items = $this->fetchFeeds(explode(" ", strip_tags($params['url'])), $params['limit']); 

        $this->set('title', pick( $params['title'], strip_tags($items[0]['channel']) ) );
        $this->set('items', $items );

        return $this->renderTemplate($params['style']);
	}
    
    /**
     * Method: fetchFeeds
     * Fetches a series of rss feeds, orders them by publication and returns the
     * item objects.
     * 
     * Parameters:
     *  feeds - Array, array of url's to fetch
     *  limit - Integer, Optional, how many items to limit to, defaults to 10
     * 
     * Return:
     *  Array - array of item object representing each rss item. 
     */
    function fetchFeeds( $feeds, $limit = 10 )
    {
        require_once( $this->galmetapost->getVar('path') . '_lib/magpierss/rss_fetch.inc');
        $cache = $this->galmetapost->getVar('cache_path');
        if ( !empty($cache) )
        {
            define('MAGPIE_CACHE_DIR',$cache . 'magpierss/');
        }

        $all = array();
        $reorder = array();

        foreach ( $feeds as $feed )
        {
            if ( $rss = fetch_rss($feed) )
            {
                foreach ( $rss->items as $item )
                {
                    $allid = count($all);
                    $item['rawdate'] = strtotime(pick( $item['pubdate'], $item['dc']['date'], $id));
                    $item['description'] = pick( $item['description'], $item['atom']['summary']);                    
                    $reorder[$allid] = $item['rawdate'];
                    $item['channel'] = $rss->channel['title'];
                    $item['channel-link'] = $rss->channel['link'];
                    $all[] = $item;
                }
            }
        }              
        
        if ( count($all) == 0 )
        {
            return false;
        }
 
        arsort($reorder);

        $items = array();
        $i = 0;
        foreach ( $reorder as $id => $date )
        {
            if ( $i++ > $limit )
            {
                break;
            }
            
            $items[] = $all[$id];
        }
        
        return $items;        
    }
}
?>