/**
 * Class: ResourceSelector
 * A page browsing widget used to browse through website pages
 * on a Post installation.
 *
 * Author: Bryan English <bryan at bluelinecity dot com>
 *
 * Usage:
 * 	var d = new ResourceSelector( 'offset/' );
 *	$('widget-div').setHTML(d.getHTML());
 *	d.update('mystartingpage');
 */
var ResourceSelector = new Class({

	initialize: function( offset )
	{
		this.current = '';
		this.listings = {};
		this.mode = 'select';
		this.showNew = false;
		this.showButton = false;
		this.showBase = false;
		this.callback = function(){alert(this.getValue());};
		this.base = window.location.host;
		this.offset = (offset?offset:'');
		this.id = ResourceSelector.dialogs.length;		
		ResourceSelector.dialogs.push( this );		
	},

	setValue: function( value )
	{
		this.current = value.replace(/(^\/)|(\/$)/,'');
	},

	getValue: function()
	{
		return this.current + (this.showNew && $(this.getId('new')) && $(this.getId('new')).getValue()?'/'+$(this.getId('new')).getValue():'');
	},

	addListing: function( base, list )
	{
		this.listings[ (base==""?"/":base) ] = list;
	},

	getListing: function( base, list )
	{
		return this.listings[ (base==""?"/":base) ];
	},

	getHTML: function()
	{
		var el = new Element('div');		
		el.adopt( this.getElement() );
		return el.innerHTML;
	},

	getElement: function()
	{
		var el = new Element('div',
		{
			'class':'resource_selector clearfix',
			id:'resource_selector['+this.id+']'
		});
		
		el.adopt(new Element('div',{'class':'path',style:'display:inline','id':this.getId('path')}));
		el.adopt(new Element('div',{'class':'name',style:'display:inline','id':this.getId('name')}));
        
		if ( this.showButton )
		{
			el.adopt(this.makeButton());
		}

		return el;
	},
    
	makeButton: function()
	{
		var el = new Element('input',
		{
			'type':'button',
			'value':'Select',
			'onclick':'ResourceSelector.dialogs['+this.id+'].callback()',
		});

		return el;
	},

	makeNew: function(  )
	{
		var selector = this;
   
		$(this.getId('name')).innerHTML = '';

		var el = new Element('input',{type:'text',id:this.getId('new'),name:this.getId('new')});
		
		$(this.getId('name')).adopt(el);
		el.focus();
	},

	makeSelect: function( options, preselect )
	{
		var selector = this;
   
		$(this.getId('name')).innerHTML = '';

		if ( options.length == 0 )
		{
			if ( this.showNew )
			{
				this.makeNew();
			}
			return;
		}
		
		var el = new Element('select',{id:this.getId('select'),name:this.getId('select')});
		el.addEvent('change',function(e){
			if ( this.selectedIndex )
			{
				if ( this.getValue() == '- New -' )
				{
					selector.makeNew();
					return;
				}
				selector.update(this.getValue());
			}
		});

		el.adopt(new Element('option',{value:''}));

		if ( this.showNew )
		{
			el.adopt(new Element('option',{value:'- New -'}).setHTML('*NEW*'));		
		}

		options.forEach(function(e,i){
			var o = new Element('option',{value:e.value}).setHTML(e.key);
			if ( preselect == e.value )
			{
				o.setProperty('selected','true');
			}
			el.adopt(o);
		});

        $(this.getId('name')).adopt(el);
		$(this.getId('name')).focus();
		
	},

	getId: function(sub)
	{
		return 'resource_selector['+this.id+']' + (sub?'/'+sub:'');
	},

	getObjectRef: function(sub)
	{
		return 'ResourceSelector.dialogs['+this.id+']';
	},

	update: function( path, preselect )
	{
		this.setValue( path );
		this.updatePath();
		this.updateSelect( preselect );
	},

	updatePath: function ()
	{	
		$(this.getId('path')).innerHTML = (this.showBase===true?this.getBaseLink() + '&nbsp;/&nbsp;':(this.showBase?this.getBaseLink(this.showBase):'')) + this.getPathLinks();
	},

	getBaseLink: function ( text )
	{
		var link = '<a href="javascript:void();" onClick="'+this.getObjectRef()+'.update(\'\')">';
		link += (text?text:this.base+this.offset.replace(/\//g," / "));
		return link + '</a>';
	},

	getPathLinks: function ()
	{
		var links = '';
		
		if ( this.current != '' )
		{
			var parts = this.current.split('/');
			var trail = '';
			parts.forEach(function(e,i){
				trail += e + '/';
				links += '<a href="javascript:void();" onClick="'+this.getObjectRef()
				+'.update(\''+ trail.replace(/\/?[^\/]+\/?$/,'') +'\',\''+ trail.replace(/\/$/,'') +'\')">'+e+'</a>&nbsp;/&nbsp;' ;	
			},this);
		}

		return links;
	},
	
	ping: function( path, yes, no )
	{
		var ajax = new Ajax( 'http://' + this.base + this.offset + "/" + path + '!ping',
			{
				method: 'get',
				onSuccess: function(e){
					if ( this.response.textContent == 1 )
					{
						yes();
					}
					else
					{
						no();
					}
				}
			}).request();			
	},
	
	updateSelect: function( preselect )
	{
	
		ready = [];

		//check cache//
		if ( this.listings[this.current] )
		{
			ready = this.listings[this.current];
			this.makeSelect(ready, preselect);
		}
		else
		{
			var self = this;
			var ajax = new Ajax( 'http://' + this.base + this.offset + "/" + (this.current==""?"":this.current+'/') + '!xml',
				{
					method: 'get',
					onSuccess: function(e){
						var options = [];
						var names = this.response.xml.getElementsByTagName('name');

						for( var c=0; c<names.length; c++)
						{
							options.push( $pick(names.item(c).textContent,names.item(c).text) );
						}

						var mask = self.current.replace(/\//g,"\\/");
						var unique = [];

						options.forEach(function(e,i){
							var key = e.replace(new RegExp("^"+mask+"\\/?([^\\/]+).*$"),'$1');
							if ( !unique[key] )
							{
								unique[key] = true;
								ready.push({key:key,value:(this.current!=''?this.current+'/'+key:key)});
							}
						},self);

						self.addListing(self.current,ready);
						self.makeSelect(ready, preselect);						
					}
				}).request();			
		}
		
	}

});

ResourceSelector.dialogs = [];

