<?php

class GalmetaPostInstaller extends PIS {          

  function page_welcome(){		      

    $this->message('Thank you for trying GalmetaPost '. $GLOBALS['GALMETAPOST_VERSION'] .'. The next few steps will assist you in installing Galmeta Post on your host.');

    $this->hr();

    if (function_exists('apache_get_modules')){
        $mods = apache_get_modules();    
        if (in_array('mod_rewrite',$mods)){
            $this->message("- Apache mod_rewrite found! Pretty urls enabled.",'passed');
            $this->set('use_mod_rewrite',true);
        } else {
            $this->message("- Apache mod_rewrite not found! Ugly urls enabled.",'failed');
            $this->set('use_mod_rewrite',false);
        }
    } else {
        $this->message("- Unable to find installed Apache modules. Please verify the use of mod_rewrite",'failed');
    }

    $this->set('path',dirname($_SERVER['SCRIPT_FILENAME']) . '/');
    $this->message('- Server Path detected: ' . $this->get('path'),'passed');

    $this->set('url','http://' . $_SERVER['SERVER_NAME'] . dirname($_SERVER['SCRIPT_NAME']) . '/');
    $this->message('- URL detected: ' . $this->get('url'),'passed');

    if (!is_writable('_files')){
        $this->message("- Files directory (_files) is not writable! But that's okay if it's intentional and you don't want to give users the ability to upload images, files, etc.",'failed');
        $this->set('files_path','');
    } else {
        $this->message('- Files directory is writable!','passed');
        $this->set('files_path','_files/');
    }                   

    if (!is_writable('_cache')){
        $this->message("- Cache directory (_cache) is not writable or non-existant! But that's okay if it's intentional and you don't want to use the cache or you've moved it above document root.",'failed');
        $this->set('cache_path','');
    } else {
        $this->message('- Cache directory is writable!','passed');
        $this->set('cache_path','_cache/');
    }  

    if (!is_writable('.')){
        $this->message('- Config file (config.php) is not writable! Which is fine,  you\'ll just have to manually copy and paste the config file.','failed');
        $this->set('__make_config',false);
    } else {
        $this->message('- Config file is writable!','passed');
        $this->set('__make_config',true);
    }

  }

  function page_license(){
     $this->input('agreement','agreement',array('required'=>true,'file'=>'gpl.txt'));
  }

  function page_config(){
    
    $this->title("Configuration Options");

    $this->input('select','db_type',array('required' => true,'label' => 'Database Type','help' => 'If your not sure check with your hosting provider.','value'=>'mysql',
        'options'=>array('fbsql','gladius','maxdb','msql','mssql','mssqlpo','mysql','mysqli','mysqlt','odbc','postgres','postgres64','postgres7','postgres8','sqlite','sqlitepo','sybase','sybase_ase')));
    $this->input('text','db_host',array('required' => true,'label' => 'Database Host','size' => 25,'help' => 'Commonly localhost'));
    $this->input('text','db_user',array('required' => true,'label' => 'Database Username','size' => 25));
    $this->input('password','db_pass',array('label' => 'Database Password','size' => 25));
    $this->input('text','db_name',array('required' => true,'label' => 'Database Name','size' => 25));

    $this->hr();

    $this->input('text','url',array('label' => 'Web Site Url','size' => 25));
    
    $this->input('text','path',array('label' => 'Server Path','size' => 25));
    
    $this->input('text','files_path',array('label' => 'Upload File Path','size' => 25));
    
    $this->input('text','cache_path',array('label' => 'Cache Path','size' => 25,
        'value' => '_cache/',
        'help' => 'This should be a world writable directory, preferably out of the doc root, that various library scripts will use to store cache data such as rss feeds and js/css compressions.'));
    
    $this->input('text','name',array('label' => 'Site Name','size' => 25,'value' => $_SERVER['SERVER_NAME'],
        'help'=>'This will be your default page on the site.'));

    $this->input('textarea','description',array('label' => 'Site Description',
        'help'=>'A description of our site, this is used in the meta tags as well as xml feeds.'));
    
    $this->input('text','table_name',array('label' => 'GalmetaPost Table Name',
        'value'=>'galmetapost','size' => 25,
        'help'=>'This will be the table name created for GalmetaPost.',
        'validation'=>'/^[a-z0-9_]+$/i',
        'validation_message'=>'Table name can only contain letters, numbers or underscores.'));
        
    $this->input('text','registry_name',array('label' => 'Registry Name','size' => 25,'value' => 'galmetapost_registry',
        'validation'=>'/^[a-z0-9_]+$/i',
        'validation_message'=>'Registry name can only contain letters, numbers or underscores.'));
        
    $this->input('text','email',array('label' => 'Administrator Email','size' => 25));
    
    $this->input('text','lock_timeout',array('label' => 'Time to lock resources when editing (minutes)','size' => 2,'value'=>2));
    
    $this->input('text','date_format',array('label' => 'PHP date format string','size'=>25,'value'=>'F j, Y (g:i a)'));
    
    $packs = glob('_lib/install/*_pack.sql');
    $themes = array();
    foreach ( $packs as $theme )
    {
        preg_match('/(_lib\/install\/([a-z]+)_pack\.sql)$/', $theme, $matches);
        $themes[ucfirst($matches[2])] = $matches[1];
    }

    $this->input('select','installation_type',array('label' => 'Installation Type','value'=>'_lib/install/blog_pack.sql','options'=>$themes,'help'=>'Static - Simple, barebones, static web page-like installation for single login, single user managed web site\n\n'));

    $this->hr();

    $this->input('boolean','safehtml',array('label' => 'Enable SafeHTML filter','value'=>true,'help'=>'This filters all resource edits through a safehtml filter to protect against XSS attacks.'));
    $this->input('boolean','use_mod_rewrite',array('label' => 'Use Apache mod_rewrite module'));
    $this->input('boolean','versioning',array('label' => 'Track resource changes (versioning)','value'=>false));
    $this->input('boolean','core_override',array('label' => 'Allow Core Overrides','value'=>true,'help'=>
        'This option allows the core resource to be overridden in folders where a core/css resource has been defined. Useful if you need to have multiple shells within the same web site.'));

  }

  function page_user_setup(){
     
         $this->input('text','admin_user',array(
             'label' => 'Administrator Username',
             'value'=>"admin",
             'validation' => '/^[a-z0-9_]+$/i',             
             'required' => true));

         $this->input('password','admin_pass',array(
             'label' => 'Administrator Password','value'=>"",
             'required' => true));

  }

  function page_installation(){    

    $this->set( 'name', preg_replace('/[^a-z0-9\-\_]/i', '', $this->get( 'name' ) ) );

    //try to connect to database//    
    $db = ADONewConnection( $this->get('db_type') );    
    
    if ( $this->test($db->Connect($this->get('db_host'),$this->get('db_user'),$this->get('db_pass'),$this->get('db_name')),
        'Unable to connect to database. Verify database settings. <br />' . $db->ErrorMsg()) ){
    
        $search = array('/\$\$\$REGISTRY_NAME\$\$\$/','/\$\$\$TABLE_NAME\$\$\$/','/\$\$\$GALMETAPOST_NAME\$\$\$/','/\$\$\$URL_PREFIX\$\$\$/','/;\s*$/');
        $replace = array($this->get('registry_name'),$this->get('table_name'),$this->get('name'),'','');

        //install database tables//
        $errors = $this->importSQLFile('_lib/install/install.sql',$db,$search,$replace,'warn'); 
        if ( is_array($errors) && count($errors) > 0 )
        {
            $this->test(false,'Unable to create table. Verify database settings/rights.<br />' . implode('<br />',$errors) );
        }

        //install registry//
        if ( !$db->Execute("INSERT INTO `".$this->get('registry_name') ."` (name,content) VALUES ('galmetapost/version','". $GLOBALS['GALMETAPOST_VERSION'] ."')") ) 
        {
            $this->test($result,'Unable to write to registry.<br />' . $db->ErrorMsg());
        }

        //install galmetapost info//
        $result = $db->Execute("INSERT INTO `".$this->get('registry_name')."` (name,content) VALUES ('galmetapost/installed','". date("F j, Y, g:i a") ."')");

        //install users//
        $result = $db->Execute("INSERT INTO `".$this->get('registry_name')."` (name,content) VALUES ('users/guest','". SHA1('') ."')");
        $result = $db->Execute("INSERT INTO `".$this->get('registry_name')."` (name,content) VALUES ('users/" . $this->get('admin_user') . "','". SHA1($this->get('admin_pass')) ."')");
        $result = $db->Execute("INSERT INTO `".$this->get('registry_name')."` (name,content) VALUES ('users/" . $this->get('admin_user') . "/settings/can_upload','true')");
        $result = $db->Execute("INSERT INTO `".$this->get('registry_name')."` (name,content) VALUES ('users/" . $this->get('admin_user') . "/settings/safehtml','false')");
        $result = $db->Execute("INSERT INTO `".$this->get('registry_name')."` (name,content) VALUES ('users/" . $this->get('admin_user') . "/settings/ToolbarSet','Admin')");

        //install default theme and other if specified//
        $errors = $this->importSQLFile($this->get('installation_type'),$db,$search,$replace,'warn');
        if ( is_array($errors) && count($errors) > 0 )
        {
            $this->test($result,'Unable to write to the database. Verity database settings/rights.<br />' . implode('<br />',$errors));
        }

        //get number of rows inserted//
        $result = $db->Execute('SELECT count(*) as cnt FROM ' . $this->get('table_name'));

        if ( $result ) 
        {
            $this->message($result->Fields('cnt') . ' rows successfully inserted.','passed');
        }
        else 
        {
            $this->message('No rows inserted. Please verify table name is valid.','failed');
        }

        if ( !$this->areErrors() ){
            if ( $this->get('__make_config') ){
                $this->message('<h3>Config file successfully created! Please adjust the permissions to write-protected config.php.</h3>');
                $this->makeConfig('\$GALMETAPOSTCONFIG[\'{$name}\'] = {$value};\n',"\$GALMETAPOSTCONFIG = array();\n",'config.php',array('admin_user','admin_pass','history','access_tokens'));
            } else {
                $this->message('<h3>Copy and paste the below php code and paste it into a file named "config.php" in the base directory you unzipped Galmeta Post too. Then click "Complete".</h3><pre>'. 
                    htmlspecialchars($this->makeConfig('\$GALMETAPOSTCONFIG[\'{$name}\'] = {$value};\n',"\$GALMETAPOSTCONFIG = array();\n",NULL,array('admin_user','admin_pass','history','access_tokens'))) .'</pre>');
            }
            
            if ( $this->get('use_mod_rewrite') ){
            	if ( preg_match('/^https?:\/\/.+(\/.+?\/.+)\/$/',$this->get('url'),$matches) ){
            		$this->message("Please update the RewriteBase directive in the .htaccess file with " . $matches[1]);
            	}
            } else {
            	$this->message("It might be a good idea to remove the .htaccess file since mod_rewrite isn't being used.");	
            }
            
            $this->cleanup();
        }                
    }
  }
}

?>