/**
 * Class: SlimPrompt
 * Fancy mootools powered replacement for dated prompt() function. 
 * 
 * Author: Bryan English <bryan at bluelinecity dot com>
 *
 * Usage:
 *  SlimPrompt.open( {
 *      title: 'A title for the box',
 *      text:  'Content of the prompt box. Any HTML can be used here.
 *      onX:   function reference, used to auto create the prompt box options and coorisponding executing code.
        beforePop: function reference, executes before prompt pops up. Can 
        afterPop: function reference, executes after prompt pops up.
        beforePip: function reference, executes before prompt pops down.
        afterPip: function reference, executes after prompt pops down.
 *  } );
 *
 * Examples:
 *  SlimPrompt.open({title:"Hello World!",text:"Hello World",onOkay:function(){this.close();}});
 *
 function NewFileDialog(){
	SlimPrompt.open({
		title:'New Page',
		width: 500,
		text:'<strong>File name?</strong><br />',
		onOkay:function(){this.close();},
		beforePop:function(){
			this.rs = new ResourceSelector("/gp/source");
			this.text += this.rs.getHTML();
		},
		afterPop:function(){
			this.rs.update("/");
		},
		afterPip:function(){
			alert(this.rs.getValue());
		}
	});
}
 */
var SlimPrompt = {    
    
    open: function(obj)
    {
        //look for onFunction's to add to prompt ask box//
        var ask = [];
        this.ini = obj;

        $each(this.ini,function(v,k){
            if ( /^on(.*)/.test(k) )
            {
                this[k] = v;
                ask.push('<a href="javascript:SlimPrompt.'+ k +'();">'+ RegExp.$1 +'</a>');
            }
        },this);

        this.ini.width = this.ini.width || 300;        
        this.ini.beforePop && this.ini.beforePop();

        //clear any previously created elements//
        $('SlimPrompt') && $('SlimPrompt').remove();
        $('SlimPromptOverlay') && $('SlimPromptOverlay').remove();        

        //add the overlay fader to body//
        $$('body')[0].adopt( new Element('div',
            {
            id:'SlimPromptOverlay',
            styles:
                {
                fontSize: 0,opacity:0,'background-color':'#000',
                position:'absolute','z-index':10,top:0,left:0,
                width:window.getScrollWidth(),height:window.getScrollHeight()
                }
            }).setHTML('&nbsp;'));
    
        //add event sink on overlay//
        $('SlimPromptOverlay').addEvent('click',function(e){e.stopPropagation();});

        //create slimprompt element//
        $$('body')[0].adopt( new Element('div',{
            id:'SlimPrompt',
            styles: 
                {
                top:window.getScrollTop()+(window.getScrollHeight()/2),
                left:window.getScrollLeft()+(window.getScrollWidth()/2),
                position:'absolute',
                'z-index':20
                }
            }).setHTML('<h1>'+this.ini.title+'</h1><div class="text">'+this.ini.text+'</div><div class="ask">'+ask.join(' | ')+'</div>'));

        //do the cool fx, fade overlay, then pop open the prompt//
        
        $('SlimPromptOverlay').effect('opacity',{duration: 250,fps:30}).start(0,0.8).chain(function(){
            $('SlimPrompt').setStyle('display','block');
            $('SlimPrompt').effects({fps:30,duration:250,transition:Fx.Transitions.Elastic.easeOut}).start({
                'width' : [0,obj.width],
                'font-size' : [0,14],
                'top': [window.getScrollTop()+(window.getScrollHeight()/2),window.getScrollTop()+(window.getScrollHeight()/3)],
                'left': [window.getScrollLeft()+(window.getScrollWidth()/2),window.getScrollLeft()+((window.getScrollWidth()-obj.width)/2)]
            }).chain(function(){
                obj.afterPop && obj.afterPop();
            });
        });
    },

    //close the topmost slimprompt//
    close: function()
    {
        var ini = this.ini;
        ini.beforePip && ini.beforePip();
        var width = $('SlimPrompt').getStyle('width').toInt();

        //do cool fx, pop out window and fade back//
        $('SlimPrompt').effects({fps:30,duration:250,transition:Fx.Transitions.Elastic.easeIn}).start({
            'width' : [width,0],
            'font-size' : [14,0],
            'padding': [5,0],
            'border-width': [$('SlimPrompt').getStyle('border-width'),0],
            'top': [window.getScrollTop()+(window.getHeight()/3),window.getScrollTop()+(window.getScrollHeight()/2)],
            'left': [window.getScrollLeft()+((window.getWidth()-width)/2),window.getScrollLeft()+(window.getScrollWidth()/2)]
        }).chain(function(){
            $('SlimPrompt').setStyle('display','none');
            $('SlimPromptOverlay').effect('opacity',{duration: 250,fps:30}).start( 0.8, 0
            ).chain(function(){
                $('SlimPrompt').remove();
                $('SlimPromptOverlay').remove();
                ini.afterPip && ini.afterPip();
            });
        });        
    }
}

